#pragma semicolon 1
#include <sourcemod>

#define CSSRPG_DB "cssrpg"
#define TBL_PLAYERS "players"
#define TBL_ITEMS "items"

// CSSRPG Database stuff
new Handle:g_hDatabase = INVALID_HANDLE;

new String:player_cols[][] = {
	"player_id INTEGER PRIMARY KEY",
	"name TEXT DEFAULT ' '",
	"steamid TEXT DEFAULT '0'",
	"level INTEGER DEFAULT '1'",
	"exp INTEGER DEFAULT '0'",
	"credits INTEGER DEFAULT '0'",
	"lastseen INTEGER DEFAULT '0'",
	"items_id INTEGER DEFAULT '-1'"
};

new String:player_col_types[][] = {"player_id", "name", "steamid", "level", "exp", "credits", "lastseen", "items_id"};

InitDatabase()
{
	decl String:sError[256];
	g_hDatabase = SQLite_UseDatabase(CSSRPG_DB, sError, sizeof(sError));
	if(g_hDatabase == INVALID_HANDLE)
		SetFailState("Error initializing database: %s", sError);
	
	new result = SQLiteTableExists(TBL_PLAYERS);
	if(!result)
	{
		decl String:sQuery[1024];
		for(new i=0;i<sizeof(player_cols);i++)
		{
			if(i)
				Format(sQuery, sizeof(sQuery), "%s, %s", sQuery, player_cols[i]);
			else
				Format(sQuery, sizeof(sQuery), "%s", player_cols[i]);
		}
		
		Format(sQuery, sizeof(sQuery), "CREATE TABLE %s (%s)", TBL_PLAYERS, sQuery);
		SQL_FastQuery(g_hDatabase, sQuery);
	}
	else if(result == 1)
	{
		decl String:sQuery[256];
		for(new i=0;i<sizeof(player_cols);i++)
		{
			if(!SQLiteColumnExists(player_col_types[i], TBL_PLAYERS))
			{
				Format(sQuery, sizeof(sQuery), "ALTER TABLE %s ADD COLUMN %s", TBL_PLAYERS, player_cols[i]);
				SQL_FastQuery(g_hDatabase, sQuery);
			}
		}
	}
	
	result = SQLiteTableExists(TBL_ITEMS);
	if(!result)
	{
		decl String:sQuery[1024];
		Format(sQuery, sizeof(sQuery), "items_id INTEGER PRIMARY KEY");
		new Handle:hItems = GetItemsList();
		new iSize = GetArraySize(hItems);
		decl Handle:hItem, String:sShortName[16];
		for(new i=0;i<iSize;i++)
		{
			hItem = GetArrayCell(hItems, i);
			GetArrayString(hItem, ITEM_SHORTNAME, sShortName, sizeof(sShortName));
			Format(sQuery, sizeof(sQuery), "%s, %s INTEGER DEFAULT '0'", sQuery, sShortName);
		}
		
		Format(sQuery, sizeof(sQuery), "CREATE TABLE %s (%s)", TBL_ITEMS, sQuery);
		SQL_FastQuery(g_hDatabase, sQuery);
	}
	else if(result == 1)
	{
		decl String:sQuery[256];
		new Handle:hItems = GetItemsList();
		new iSize = GetArraySize(hItems);
		decl Handle:hItem, String:sShortName[16];
		for(new i=0;i<iSize;i++)
		{
			hItem = GetArrayCell(hItems, i);
			GetArrayString(hItem, ITEM_SHORTNAME, sShortName, sizeof(sShortName));
			if(!SQLiteColumnExists(sShortName, TBL_ITEMS))
			{
				Format(sQuery, sizeof(sQuery), "ALTER TABLE %s ADD COLUMN %s INTEGER DEFAULT '0'", TBL_ITEMS, sShortName);
				SQL_FastQuery(g_hDatabase, sQuery);
			}
		}
	}
}

DatabaseMaid()
{
	if(!GetConVarBool(g_hCVSaveData))
		return;
	
	decl String:sQuery[256];
	
	/* Delete players who are Level 1 and haven't played for 3 days */
	Format(sQuery, sizeof(sQuery), "DELETE FROM %s WHERE level <= '1' AND lastseen <= '%d'", TBL_PLAYERS, GetTime()-259200);
	SQL_FastQuery(g_hDatabase, sQuery);
	
	if(GetConVarInt(g_hCVPlayerExpire) > 0)
	{
		Format(sQuery, sizeof(sQuery), "SELECT items_id FROM %s WHERE lastseen <= '%d'", TBL_PLAYERS, GetTime()-(86400*GetConVarInt(g_hCVPlayerExpire)));
		new Handle:hResult = SQL_Query(g_hDatabase, sQuery);
		if(hResult != INVALID_HANDLE)
		{
			while(SQL_MoreRows(hResult))
			{
				SQL_FetchRow(hResult);
				Format(sQuery, sizeof(sQuery), "DELETE FROM %s WHERE items_id = '%d'", TBL_ITEMS, SQL_FetchInt(hResult, 0));
				SQL_FastQuery(g_hDatabase, sQuery);
				Format(sQuery, sizeof(sQuery), "DELETE FROM %s WHERE items_id = '%d'", TBL_PLAYERS, SQL_FetchInt(hResult, 0));
				SQL_FastQuery(g_hDatabase, sQuery);
			}
			CloseHandle(hResult);
		}
		else
		{
			LogError("DatabaseMaid: player expire query failed");
		}
	}
	
	Format(sQuery, sizeof(sQuery), "VACUUM %s", TBL_PLAYERS);
	SQL_FastQuery(g_hDatabase, sQuery);
	Format(sQuery, sizeof(sQuery), "VACUUM %s", TBL_ITEMS);
	SQL_FastQuery(g_hDatabase, sQuery);
}

/* A very cheap way of doing things but there is no other alternative */
#define NO_SUCH_TBL "no such table"
SQLiteTableExists(String:table[])
{
	decl String:sQuery[64];
	Format(sQuery, sizeof(sQuery), "SELECT * FROM %s", table);
	if(SQL_FastQuery(g_hDatabase, sQuery))
		return 1;
	
	decl String:sError[256];
	SQL_GetError(g_hDatabase, sError, sizeof(sError));
	if(StrContains(sError, NO_SUCH_TBL, false) != -1)
		return 0;
	
	LogError("Error checking if table \"%s\" exists: %s", table, sError);
	return -1;
}

#define NO_SUCH_COL "no such column"
SQLiteColumnExists(String:col[], String:table[])
{
	decl String:sQuery[64];
	Format(sQuery, sizeof(sQuery), "SELECT %s FROM %s", col, table);
	if(SQL_FastQuery(g_hDatabase, sQuery))
		return 1;
	
	decl String:sError[256];
	SQL_GetError(g_hDatabase, sError, sizeof(sError));
	if(StrContains(sError, NO_SUCH_COL, false) != -1)
		return 0;
	
	LogError("Error checking if column \"%s\".\"%s\" exists: %s", table, col, sError);
	return -1;
}