#pragma semicolon 1
#include <sourcemod>

#define ITEM_INDEX 0
#define ITEM_NAME 1
#define ITEM_SHORTNAME 2
#define ITEM_ENABLE 3
#define ITEM_MAXLEVEL 4
#define ITEM_MAXLEVELBARRIER 5
#define ITEM_STARTCOST 6
#define ITEM_INCCOST 7
#define ITEM_BUYCALLBACK 8
#define ITEM_SELLCALLBACK 9
#define ITEM_ENABLE_CVAR 10
#define ITEM_MAXLEVEL_CVAR 11
#define ITEM_STARTCOST_CVAR 12
#define ITEM_INCCOST_CVAR 13

#include "items/rpgi_regen.sp"
#include "items/rpgi_hbonus.sp"
#include "items/rpgi_fnade.sp"
#include "items/rpgi_fpistol.sp"
#include "items/rpgi_icestab.sp"
#include "items/rpgi_impulse.sp"
#include "items/rpgi_ljump.sp"
#include "items/rpgi_medic.sp"
#include "items/rpgi_resup.sp"
#include "items/rpgi_stealth.sp"
#include "items/rpgi_vampire.sp"
#include "items/rpgi_denial.sp"

new Handle:g_hItems;

RegisterItems()
{
	g_hItems = CreateArray();
	
	Item_Regeneration_PreInit();
	Item_HealthPlus_PreInit();
	Item_Resupply_PreInit();
	Item_Vampire_PreInit();
	Item_Stealth_PreInit();
	Item_LJump_PreInit();
	Item_FNade_PreInit();
	Item_IceStab_PreInit();
	Item_FPistol_PreInit();
	Item_Impulse_PreInit();
	Item_Denial_PreInit();
	Item_Medic_PreInit();
	
}

InitItems()
{
	Regeneration_Init();
	Medic_Init();
	Resupply_Init();
	Stealth_Init();
}

ShutdownItems()
{
	Regeneration_Shutdown();
	Medic_Shutdown();
	Resupply_Shutdown();
	Stealth_Shutdown();
}

Handle:GetItemsList()
{
	return g_hItems;
}

Handle:RegisterItemType(String:name[], String:shortname[], maxlevelbarrier, bool:bDefaultEnable, iDefaultMaxLevel, iDefaultStartCost, iDefaultCostInc, Function:buycb, Function:sellcb)
{
	new Handle:hItem = CreateArray(ByteCountToCells(128));
	new iIndex = PushArrayCell(g_hItems, hItem);
	PushArrayCell(hItem, iIndex);
	PushArrayString(hItem, name);
	PushArrayString(hItem, shortname);
	PushArrayCell(hItem, bDefaultEnable); // enable
	PushArrayCell(hItem, iDefaultMaxLevel); // maxlevel
	PushArrayCell(hItem, maxlevelbarrier);
	PushArrayCell(hItem, iDefaultStartCost); // start_cost
	PushArrayCell(hItem, iDefaultCostInc); // inc_cost
	PushArrayCell(hItem, buycb);
	PushArrayCell(hItem, sellcb);
	
	decl String:sCvarName[64], String:sCvarDescription[128], String:sValue[16];
	
	// Register convars
	Format(sCvarName, sizeof(sCvarName), "cssrpg_%s_enable", shortname);
	Format(sCvarDescription, sizeof(sCvarDescription), "Sets the %s item to enabled (1) or disabled (0)", name);
	IntToString(_:bDefaultEnable, sValue, sizeof(sValue));
	new Handle:hCvar = CreateConVar(sCvarName, sValue, sCvarDescription, 0, true, 0.0, true, 1.0);
	HookConVarChange(hCvar, ConVar_ItemChanged);
	PushArrayCell(hItem, hCvar);
	
	// TODO: Handle maxlevel > maxlevelbarrier etc. rpgi.cpp CVARItemMaxLvl!
	Format(sCvarName, sizeof(sCvarName), "cssrpg_%s_maxlevel", shortname);
	Format(sCvarDescription, sizeof(sCvarDescription), "%s item maximum level", name);
	IntToString(iDefaultMaxLevel, sValue, sizeof(sValue));
	hCvar = CreateConVar(sCvarName, sValue, sCvarDescription, 0, true, 1.0);
	HookConVarChange(hCvar, ConVar_ItemMaxLevelChanged);
	PushArrayCell(hItem, hCvar);
	
	Format(sCvarName, sizeof(sCvarName), "cssrpg_%s_cost", shortname);
	Format(sCvarDescription, sizeof(sCvarDescription), "%s item start cost", name);
	IntToString(iDefaultStartCost, sValue, sizeof(sValue));
	hCvar = CreateConVar(sCvarName, sValue, sCvarDescription, 0, true, 0.0);
	HookConVarChange(hCvar, ConVar_ItemChanged);
	PushArrayCell(hItem, hCvar);
	
	Format(sCvarName, sizeof(sCvarName), "cssrpg_%s_icost", shortname);
	Format(sCvarDescription, sizeof(sCvarDescription), "%s item cost increment for each level", name);
	IntToString(iDefaultCostInc, sValue, sizeof(sValue));
	hCvar = CreateConVar(sCvarName, sValue, sCvarDescription, 0, true, 0.0);
	HookConVarChange(hCvar, ConVar_ItemChanged);
	PushArrayCell(hItem, hCvar);
	
	return hItem;
}

// Cache the new config value
public ConVar_ItemChanged(Handle:convar, const String:oldValue[], const String:newValue[])
{
	new iSize = GetArraySize(g_hItems);
	new Handle:hItem;
	for(new i=0;i<iSize;i++)
	{
		hItem = GetArrayCell(g_hItems, i);
		
		if(Handle:GetArrayCell(hItem, ITEM_ENABLE_CVAR) == convar)
		{
			SetArrayCell(hItem, ITEM_ENABLE, GetConVarBool(convar));
			break;
		}
		else if(Handle:GetArrayCell(hItem, ITEM_MAXLEVEL_CVAR) == convar)
		{
			SetArrayCell(hItem, ITEM_MAXLEVEL, GetConVarInt(convar));
			break;
		}
		else if(Handle:GetArrayCell(hItem, ITEM_STARTCOST_CVAR) == convar)
		{
			SetArrayCell(hItem, ITEM_STARTCOST, GetConVarInt(convar));
			break;
		}
		else if(Handle:GetArrayCell(hItem, ITEM_INCCOST_CVAR) == convar)
		{
			SetArrayCell(hItem, ITEM_INCCOST, GetConVarInt(convar));
			break;
		}
		
	}
}

public ConVar_ItemMaxLevelChanged(Handle:convar, const String:oldValue[], const String:newValue[])
{
	new iSize = GetArraySize(g_hItems);
	new Handle:hItem, iIndex;
	for(iIndex=0;iIndex<iSize;iIndex++)
	{
		hItem = GetArrayCell(g_hItems, iIndex);
		
		if(Handle:GetArrayCell(hItem, ITEM_MAXLEVEL_CVAR) == convar)
			break;
	}
	
	new iNewMaxLevel = GetConVarInt(convar);
	new iMaxLevelBarrier = GetArrayCell(hItem, ITEM_MAXLEVELBARRIER);
	
	if(!GetConVarBool(g_hCVIgnoreLevelBarrier) && iNewMaxLevel > iMaxLevelBarrier)
		iNewMaxLevel = iMaxLevelBarrier;
	
	SetArrayCell(hItem, ITEM_MAXLEVEL, iNewMaxLevel);
	
	for(new i=1;i<=MaxClients;i++)
	{
		if(!IsClientInGame(i))
			continue;
		
		while(GetPlayerItemLevel(i, iIndex) > iNewMaxLevel)
		{
			SetPlayerCredits(i, GetPlayerCredits(i) + GetItemCost(iIndex, GetPlayerItemLevel(i, iIndex)));
			TakeItem(i, iIndex);
		}
	}
}

stock GetItemIndexByShortname(String:sShortName[])
{
	new iSize = GetArraySize(g_hItems);
	new Handle:hItem;
	decl String:sBuffer[16];
	for(new i=0;i<iSize;i++)
	{
		hItem = GetArrayCell(g_hItems, i);
		
		GetArrayString(hItem, ITEM_SHORTNAME, sBuffer, sizeof(sBuffer));
		if(StrEqual(sBuffer, sShortName, false))
			return i;
	}
	return -1;
}

stock Handle:GetItemByShortname(String:sShortName[])
{
	new iSize = GetArraySize(g_hItems);
	new Handle:hItem;
	decl String:sBuffer[16];
	for(new i=0;i<iSize;i++)
	{
		hItem = GetArrayCell(g_hItems, i);
		
		GetArrayString(hItem, ITEM_SHORTNAME, sBuffer, sizeof(sBuffer));
		if(StrEqual(sBuffer, sShortName, false))
			return hItem;
	}
	return INVALID_HANDLE;
}

stock Handle:GetItemByIndex(iItemIndex)
{
	return GetArrayCell(g_hItems, iItemIndex);
}

stock GetItemCost(iItemIndex, iLevel)
{
	new Handle:hItem = GetArrayCell(g_hItems, iItemIndex);
	if(iLevel <= 1)
		return GetArrayCell(hItem, ITEM_STARTCOST);
	else
		return GetArrayCell(hItem, ITEM_STARTCOST) + GetArrayCell(hItem, ITEM_INCCOST) * (iLevel-1);
}

stock GetItemSale(iItemIndex, iLevel)
{
	new iCost = GetItemCost(iItemIndex, iLevel);
	
	new Float:fSalePercent = GetConVarFloat(g_hCVSalePercent);
	if(fSalePercent == 1.0)
		return iCost;
	
	if(iLevel <= 1)
		return iCost;
	
	new iSale = RoundToFloor(float(iCost) * (fSalePercent > 1.0 ? (fSalePercent/100.0) : fSalePercent) + 0.5);
	new iCreditsInc = GetConVarInt(g_hCVCreditsInc);
	if(iCreditsInc <= 1)
		return iSale;
	else
		iSale = (iSale + RoundToFloor(float(iCreditsInc)/2.0)) / iCreditsInc * iCreditsInc;
	
	if(iSale > iCost)
		return iCost;
	
	return iSale;
}

/*stock SetEntityRenderColorNoAlpha(entity, r, g, b)
{
	new iColor = GetEntProp(entity, Prop_Send, "m_clrRender");
	
	iColor &= ~0xFF;
	iColor |= r;
	
	iColor &= ~0xFF00;
	iColor |= (g<<8);
	
	iColor &= ~0xFF0000;
	iColor |= (b<<16);
	
	SetEntProp(entity, Prop_Send, "m_clrRender", iColor);
}

stock SetEntityRenderColorAlpha(entity, a)
{
	new iColor = GetEntProp(entity, Prop_Send, "m_clrRender");
	
	iColor &= ~0xFF000000;
	iColor |= (a<<24);
	
	SetEntProp(entity, Prop_Send, "m_clrRender", iColor);
}*/