#pragma semicolon 1
#include <sourcemod>

new g_iPlayerLevel[MAXPLAYERS+1];
new g_iPlayerExperience[MAXPLAYERS+1];
new g_iPlayerCredits[MAXPLAYERS+1];

new Handle:g_hPlayerItemsLevel[MAXPLAYERS+1];

new g_iPlayerDBId[MAXPLAYERS+1] = {-1,...};
new g_iPlayerDBItemID[MAXPLAYERS+1] = {-1,...};
new bool:g_bTriedToLoadData[MAXPLAYERS+1] = {false,...};

InitPlayer(client)
{
	g_iPlayerLevel[client] = 1;
	g_iPlayerExperience[client] = 0;
	g_iPlayerCredits[client] = GetConVarInt(g_hCVCreditsStart);
	
	g_iPlayerDBId[client] = -1;
	g_iPlayerDBItemID[client] = -1;
	g_bTriedToLoadData[client] = false;
	
	g_hPlayerItemsLevel[client] = CreateArray();
	new Handle:hItems = GetItemsList();
	new iNumItems = GetArraySize(hItems);
	for(new i=0;i<iNumItems;i++)
	{
		PushArrayCell(g_hPlayerItemsLevel[client], 0); // level 0 for all items
	}
}

AddPlayer(client, const String:auth[])
{
	if(!GetConVarBool(g_hCVEnable))
		return;
	
	if(IsFakeClient(client))
		return;
	
	decl String:sQuery[256];
	
	if(GetConVarBool(g_hCVSteamIDSave))
	{
		Format(sQuery, sizeof(sQuery), "SELECT player_id, items_id, level, exp, credits FROM %s WHERE steamid = '%s' ORDER BY level DESC LIMIT 1", TBL_PLAYERS, auth);
	}
	else
	{
		decl String:sName[MAX_NAME_LENGTH], String:sNameEscaped[MAX_NAME_LENGTH*2+1];
		GetClientName(client, sName, sizeof(sName));
		SQL_EscapeString(g_hDatabase, sName, sNameEscaped, sizeof(sNameEscaped));
		Format(sQuery, sizeof(sQuery), "SELECT player_id, items_id, level, exp, credits FROM %s WHERE name = '%s' AND steamid = '%s' ORDER BY level DESC LIMIT 1", TBL_PLAYERS, sNameEscaped, auth);
	}
	
	SQL_TQuery(g_hDatabase, SQL_GetPlayerInfo, sQuery, GetClientUserId(client));
}

InsertPlayer(client)
{
	if(!GetConVarBool(g_hCVEnable) || !GetConVarBool(g_hCVSaveData))
		return;
	
	decl String:sQuery[512];
	decl String:sName[MAX_NAME_LENGTH], String:sNameEscaped[MAX_NAME_LENGTH*2+1];
	GetClientName(client, sName, sizeof(sName));
	SQL_EscapeString(g_hDatabase, sName, sNameEscaped, sizeof(sNameEscaped));
	
	decl String:sSteamID[32], String:sSteamIDEscaped[65];
	GetClientAuthString(client, sSteamID, sizeof(sSteamID));
	SQL_EscapeString(g_hDatabase, sSteamID, sSteamIDEscaped, sizeof(sSteamIDEscaped));
	
	Format(sQuery, sizeof(sQuery), "INSERT INTO %s (name, steamid, level, exp, credits, lastseen) VALUES ('%s', '%s', '%d', '%d', '%d', '%d')",
		TBL_PLAYERS, sNameEscaped, sSteamIDEscaped, g_iPlayerLevel[client], g_iPlayerExperience[client], g_iPlayerCredits[client], GetTime());
	
	SQL_TQuery(g_hDatabase, SQL_InsertPlayer, sQuery, GetClientUserId(client));
}

SaveData(client)
{
	if(!GetConVarBool(g_hCVEnable) || !GetConVarBool(g_hCVSaveData))
		return;
	
	if(IsFakeClient(client))
		return;
	
	// We're still in the process of loading this client's info from the db. Wait for it..
	if(!g_bTriedToLoadData[client])
		return;
	
	if(g_iPlayerDBId[client] < 0 || g_iPlayerDBItemID[client] < 0)
	{
		InsertPlayer(client);
		return;
	}
	
	decl String:sQuery[1024];
	Format(sQuery, sizeof(sQuery), "UPDATE %s SET level = '%d', exp = '%d', credits = '%d', lastseen = '%d' WHERE player_id = '%d'", TBL_PLAYERS, g_iPlayerLevel[client], g_iPlayerExperience[client], g_iPlayerCredits[client], GetTime(), g_iPlayerDBId[client]);
	SQL_TQuery(g_hDatabase, SQL_DoNothing, sQuery);
	
	// Save item levels
	new Handle:hItems = GetItemsList();
	new iNumItems = GetArraySize(hItems);
	decl String:sShortName[16], Handle:hItem;
	for(new i=0;i<iNumItems;i++)
	{
		hItem = GetArrayCell(hItems, i);
		GetArrayString(hItem, ITEM_SHORTNAME, sShortName, sizeof(sShortName));
		
		if(i)
			Format(sQuery, sizeof(sQuery), "%s, %s = '%d'", sQuery, sShortName, GetArrayCell(g_hPlayerItemsLevel[client], i));
		else
			Format(sQuery, sizeof(sQuery), "%s = '%d'", sShortName, GetArrayCell(g_hPlayerItemsLevel[client], i));
	}
	Format(sQuery, sizeof(sQuery), "UPDATE %s SET %s WHERE items_id = '%d'", TBL_ITEMS, sQuery, g_iPlayerDBItemID[client]);
	SQL_TQuery(g_hDatabase, SQL_DoNothing, sQuery);
}

SaveAllPlayers()
{
	if(!GetConVarBool(g_hCVEnable) || !GetConVarBool(g_hCVSaveData))
		return;
	
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && IsClientAuthorized(i))
			SaveData(i);
	}
}

ResetStats(client)
{
	DebugMsg("Stats have been reset for player: %N", client);
	
	new iSize = GetArraySize(g_hPlayerItemsLevel[client]);
	new Handle:hItems = GetItemsList();
	new Handle:hItem;
	for(new i=0;i<iSize;i++)
	{
		SetArrayCell(g_hPlayerItemsLevel[client], i, 0);
		
		hItem = GetArrayCell(hItems, i);
		Call_StartFunction(INVALID_HANDLE, Function:GetArrayCell(hItem, ITEM_SELLCALLBACK));
		Call_PushCell(client);
		Call_Finish();
	}
	
	g_iPlayerLevel[client] = 1;
	g_iPlayerExperience[client] = 0;
	g_iPlayerCredits[client] = GetConVarInt(g_hCVCreditsStart);
}

RemovePlayer(client)
{
	ResetStats(client);
	g_iPlayerDBId[client] = -1;
	g_iPlayerDBItemID[client] = -1;
	g_bTriedToLoadData[client] = false;
}

bool:GiveItem(client, iItemIndex)
{
	new Handle:hItem = GetArrayCell(GetItemsList(), iItemIndex);
	new iCurrentLevel = GetArrayCell(g_hPlayerItemsLevel[client], iItemIndex);
	
	if(iCurrentLevel >= GetArrayCell(hItem, ITEM_MAXLEVEL))
		return false;
	
	iCurrentLevel++;
	SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, iCurrentLevel);
	
	new bool:bResult;
	Call_StartFunction(INVALID_HANDLE, Function:GetArrayCell(hItem, ITEM_BUYCALLBACK));
	Call_PushCell(client);
	Call_Finish(bResult);
	
	if(!bResult)
	{
		iCurrentLevel--;
		SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, iCurrentLevel);
		return false;
	}
	
	return true;
}

bool:TakeItem(client, iItemIndex)
{
	new Handle:hItem = GetArrayCell(GetItemsList(), iItemIndex);
	new iCurrentLevel = GetArrayCell(g_hPlayerItemsLevel[client], iItemIndex);
	
	if(iCurrentLevel <= 0)
		return false;
	
	iCurrentLevel--;
	SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, iCurrentLevel);
	
	new bool:bResult;
	Call_StartFunction(INVALID_HANDLE, Function:GetArrayCell(hItem, ITEM_SELLCALLBACK));
	Call_PushCell(client);
	Call_Finish(bResult);
	
	if(!bResult)
	{
		iCurrentLevel++;
		SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, iCurrentLevel);
		return false;
	}
	
	return true;
}

bool:BuyItem(client, iItemIndex)
{
	new Handle:hItem = GetArrayCell(GetItemsList(), iItemIndex);
	new iCurrentLevel = GetArrayCell(g_hPlayerItemsLevel[client], iItemIndex);
	
	if(iCurrentLevel >= GetArrayCell(hItem, ITEM_MAXLEVEL))
		return false;
	
	new iCost = GetItemCost(iItemIndex, iCurrentLevel+1);
	
	if(iCost > g_iPlayerCredits[client])
		return false;
	
	decl String:sShortName[16];
	GetArrayString(hItem, ITEM_SHORTNAME, sShortName, sizeof(sShortName));
	DebugMsg("%N bought item %s Lvl %d", client, sShortName, iCurrentLevel+1);
	
	if(!GiveItem(client, iItemIndex))
		return false;
	
	g_iPlayerCredits[client] -= iCost;
	
	return true;
}

bool:SellItem(client, iItemIndex)
{
	new Handle:hItem = GetArrayCell(GetItemsList(), iItemIndex);
	new iCurrentLevel = GetArrayCell(g_hPlayerItemsLevel[client], iItemIndex);
	
	if(iCurrentLevel <= 0)
		return false;
	
	decl String:sShortName[16];
	GetArrayString(hItem, ITEM_SHORTNAME, sShortName, sizeof(sShortName));
	DebugMsg("%N sold item %s Lvl %d", client, sShortName, iCurrentLevel);
	
	if(!TakeItem(client, iItemIndex))
		return false;
	
	g_iPlayerCredits[client] += GetItemSale(iItemIndex, iCurrentLevel);
	
	return true;
}

CheckItemMaxLevels(client)
{
	new Handle:hItems = GetItemsList();
	new iNumItems = GetArraySize(hItems);
	new Handle:hItem, iMaxLevel, iCurrentLevel;
	for(new i=0;i<iNumItems;i++)
	{
		hItem = GetArrayCell(hItems, i);
		iMaxLevel = GetArrayCell(hItem, ITEM_MAXLEVEL);
		iCurrentLevel = GetArrayCell(g_hPlayerItemsLevel[client], i);
		while(iCurrentLevel > iMaxLevel)
		{
			/* Give player their credits back */
			/* TakeItem isn't necessary since the player hasn't even been completely added yet */
			g_iPlayerCredits[client] += GetItemCost(i, iCurrentLevel--);
		}
		SetArrayCell(g_hPlayerItemsLevel[client], i, iCurrentLevel);
	}
}

GetPlayerItemLevel(client, iItemIndex)
{
	return GetArrayCell(g_hPlayerItemsLevel[client], iItemIndex);
}

SetPlayerItemLevel(client, iItemIndex, iLevel)
{
	new iOldLevel = GetPlayerItemLevel(client, iItemIndex);
	
	SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, iLevel);
	
	if(iOldLevel < iLevel)
	{
		new bool:bResult;
		Call_StartFunction(INVALID_HANDLE, Function:GetArrayCell(GetItemByIndex(iItemIndex), ITEM_BUYCALLBACK));
		Call_PushCell(client);
		Call_Finish(bResult);
	}
	else
	{
		new bool:bResult;
		Call_StartFunction(INVALID_HANDLE, Function:GetArrayCell(GetItemByIndex(iItemIndex), ITEM_SELLCALLBACK));
		Call_PushCell(client);
		Call_Finish(bResult);
	}
}

GetPlayerCredits(client)
{
	return g_iPlayerCredits[client];
}

SetPlayerCredits(client, credits)
{
	if(credits < 0)
		credits = 0;
	g_iPlayerCredits[client] = credits;
}

GetPlayerLevel(client)
{
	return g_iPlayerLevel[client];
}

SetPlayerLevel(client, level)
{
	if(level < 1)
		level = 1;
	g_iPlayerLevel[client] = level;
}

GetPlayerExperience(client)
{
	return g_iPlayerExperience[client];
}

SetPlayerExperience(client, exp)
{
	if(exp < 0)
		exp = 0;
	g_iPlayerExperience[client] = exp;
}

// Have bots buy upgrades too :)
BotPickUpgrade(client)
{
	new iItemsBought, iCost;
	
	new Handle:hItems = GetItemsList();
	new iNumItems = GetArraySize(hItems);
	new Handle:hItem;
	
	while(GetPlayerCredits(client))
	{
		iItemsBought = 0;
		for(new i=0;i<iNumItems;i++)
		{
			hItem = GetArrayCell(hItems, i);
			
			if(!GetArrayCell(hItem, ITEM_ENABLE))
				continue;
			
			iCost = GetItemCost(i, GetPlayerItemLevel(client, i)+1);
			if(GetPlayerCredits(client) >= iCost)
			{
				BuyItem(client, i);
				iItemsBought++;
			}
		}
		if(!iItemsBought)
			break; /* Couldn't afford anything */
	}
}

stock GetClientByPlayerID(iPlayerId)
{
	for(new i=1;i<=MaxClients;i++)
	{
		if(IsClientInGame(i) && g_iPlayerDBId[i] == iPlayerId)
			return i;
	}
	return -1;
}

public SQL_GetPlayerInfo(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return;
	
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Unable to load player data (%s)", error);
		g_bTriedToLoadData[client] = true;
		return;
	}
	
	// First time this player connects?
	if(SQL_GetRowCount(hndl) == 0)
	{
		InsertPlayer(client);
		return;
	}
	
	SQL_FetchRow(hndl);
	
	g_iPlayerDBId[client] = SQL_FetchInt(hndl, 0);
	g_iPlayerDBItemID[client] = SQL_FetchInt(hndl, 1);
	g_iPlayerLevel[client] = SQL_FetchInt(hndl, 2);
	g_iPlayerExperience[client] = SQL_FetchInt(hndl, 3);
	new iTempCredits = SQL_FetchInt(hndl, 4);
	g_iPlayerCredits[client] = (iTempCredits >= 0? iTempCredits : 0);
	
	UpdatePlayerRank(client);
	UpdateRankCount();
	
	/* Player Items */
	decl String:sQuery[128];
	Format(sQuery, sizeof(sQuery), "SELECT * FROM %s WHERE items_id = '%d'", TBL_ITEMS, g_iPlayerDBItemID[client]);
	SQL_TQuery(g_hDatabase, SQL_GetPlayerItems, sQuery, userid);
}

public SQL_GetPlayerItems(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return;
	
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Unable to load item data (%s)", error);
		g_bTriedToLoadData[client] = true;
		return;
	}
	
	// Player isn't fully registred?!
	if(SQL_GetRowCount(hndl) == 0)
	{
		LogError("Player %N is registred, but doesn't have an items table entry?", client);
		g_bTriedToLoadData[client] = true;
		return;
	}
	
	new iNumFields = SQL_GetFieldCount(hndl);
	decl String:sFieldName[16], iItemIndex;
	for(new i=0;i<iNumFields;i++)
	{
		SQL_FieldNumToName(hndl, i, sFieldName, sizeof(sFieldName));
		iItemIndex = GetItemIndexByShortname(sFieldName);
		if(iItemIndex == -1)
			continue;
		
		SetArrayCell(g_hPlayerItemsLevel[client], iItemIndex, SQL_FetchInt(hndl, i));
	}
	
	g_bTriedToLoadData[client] = true;
	
	CheckItemMaxLevels(client);
	
	Stealth_SetVisibilities();
}

public SQL_InsertPlayer(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return;
	
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Unable to insert player info (%s)", error);
		g_bTriedToLoadData[client] = true;
		return;
	}
	
	g_iPlayerDBId[client] = SQL_GetInsertId(owner);
	
	UpdatePlayerRank(client);
	UpdateRankCount();
	
	// Insert item level info
	decl String:sFields[1024], String:sValues[1024], String:sBuffer[16];
	new Handle:hItems = GetItemsList();
	new iNumItems = GetArraySize(hItems);
	new Handle:hItem;
	for(new i=0;i<iNumItems;i++)
	{
		hItem = GetArrayCell(hItems, i);
		GetArrayString(hItem, ITEM_SHORTNAME, sBuffer, sizeof(sBuffer));
		if(i)
		{
			Format(sFields, sizeof(sFields), "%s, %s", sFields, sBuffer);
			Format(sValues, sizeof(sValues), "%s, '%d'", sValues, GetArrayCell(g_hPlayerItemsLevel[client], i));
		}
		else
		{
			Format(sFields, sizeof(sFields), "%s", sBuffer);
			Format(sValues, sizeof(sValues), "'%d'", GetArrayCell(g_hPlayerItemsLevel[client], i));
		}
	}
	
	decl String:sQuery[2048];
	Format(sQuery, sizeof(sQuery), "INSERT INTO %s (%s) VALUES (%s)", TBL_ITEMS, sFields, sValues);
	SQL_TQuery(g_hDatabase, SQL_InsertPlayerItems, sQuery, userid);
}

public SQL_InsertPlayerItems(Handle:owner, Handle:hndl, const String:error[], any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return;
	
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Unable to insert player items info (%s)", error);
		g_bTriedToLoadData[client] = true;
		return;
	}
	
	g_iPlayerDBItemID[client] = SQL_GetInsertId(owner);
	
	decl String:sQuery[128];
	Format(sQuery, sizeof(sQuery), "UPDATE %s SET items_id = '%d' WHERE player_id = '%d'", TBL_PLAYERS, g_iPlayerDBItemID[client], g_iPlayerDBId[client]);
	SQL_TQuery(g_hDatabase, SQL_DoNothing, sQuery);
	
	g_bTriedToLoadData[client] = true;
}

public SQL_DoNothing(Handle:owner, Handle:hndl, const String:error[], any:data)
{
	if(hndl == INVALID_HANDLE || strlen(error) > 0)
	{
		LogError("Error executing query: %s", error);
	}
}