#pragma semicolon 1
#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <smlib>

enum DenialEquipmentTypes {
	HE = 0,
	FlashBang,
	Smoke,
	Defusekit,
	Nightvision,
	Denial_Num
};

new Handle:g_hDenialConfig;
new Handle:g_hCVDenialRestrict;

new bool:g_bDenialPlayerWasDead[MAXPLAYERS+1];
new Handle:g_hDenialStripTimer[MAXPLAYERS+1] = {INVALID_HANDLE,...};

new String:g_sDenialPrimary[MAXPLAYERS+1][64];
new String:g_sDenialSecondary[MAXPLAYERS+1][64];
new g_eDenialEquipment[MAXPLAYERS+1][DenialEquipmentTypes];

Item_Denial_PreInit()
{
	g_hDenialConfig = RegisterItemType("Denial", "denial", 3, true, 3, 25, 50, Denial_Buy, Denial_Sell);
	g_hCVDenialRestrict = CreateConVar("cssrpg_denial_restrict", "", "Space delimited list of restricted weapons (e.g. awp g3sg1 m249)");
}

public bool:Denial_Buy(client)
{
	return true;
}

public bool:Denial_Sell(client)
{
	return true;
}

Denial_ResetClient(client)
{
	g_bDenialPlayerWasDead[client] = false;
	Denial_ResetItems(client);
}

Denial_ResetItems(client)
{
	g_sDenialPrimary[client][0] = '\0';
	g_sDenialSecondary[client][0] = '\0';
	ClearHandle(g_hDenialStripTimer[client]);
	
	for(new i=0;i<_:Denial_Num;i++)
	{
		g_eDenialEquipment[client][i] = 0;
	}
}

Denial_OnClientPutInServer(client)
{
	SDKHook(client, SDKHook_WeaponEquipPost, Denial_Hook_WeaponEquipPost);
}

public Action:Denial_Hook_WeaponEquipPost(client, weapon)
{
	/* Reset player Denial data while Denial is disabled */
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hDenialConfig, ITEM_ENABLE))
	{
		Denial_ResetClient(client);
		return Plugin_Continue;
	}
	
	if(weapon <= MaxClients)
		return Plugin_Continue;
	
	// Ignore pickup of weapons before spawn
	if(g_bDenialPlayerWasDead[client])
		return Plugin_Continue;
	
	if(weapon == GetPlayerWeaponSlot(client, 0))
		GetEntityClassname(weapon, g_sDenialPrimary[client], sizeof(g_sDenialPrimary[]));
	else if(weapon == GetPlayerWeaponSlot(client, 1))
		GetEntityClassname(weapon, g_sDenialSecondary[client], sizeof(g_sDenialSecondary[]));
	
	return Plugin_Continue;
}

Denial_OnPlayerSpawn(client)
{
	/* Reset player Denial data while Denial is disabled */
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hDenialConfig, ITEM_ENABLE))
	{
		Denial_ResetClient(client);
		return;
	}
	
	// Only change weapons, if he was dead.
	if(!g_bDenialPlayerWasDead[client])
		return;
	
	g_bDenialPlayerWasDead[client] = false;
	
	// Strip weapons
	ClearHandle(g_hDenialStripTimer[client]);
	g_hDenialStripTimer[client] = CreateTimer(0.1, Denial_Timer_StripPlayer, GetClientUserId(client), TIMER_FLAG_NO_MAPCHANGE);
}

Denial_OnPlayerDeath(client)
{
	/* Reset player Denial data while Denial is disabled */
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hDenialConfig, ITEM_ENABLE))
	{
		Denial_ResetClient(client);
		return;
	}
	
	g_bDenialPlayerWasDead[client] = true;
}


Denial_OnItemPickup(client, String:sItem[])
{
	/* Reset player Denial data while Denial is disabled */
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hDenialConfig, ITEM_ENABLE))
	{
		Denial_ResetClient(client);
		return;
	}
	
	if(StrEqual(sItem, "flashbang"))
		g_eDenialEquipment[client][FlashBang] = 1;
	else if(StrEqual(sItem, "hegrenade"))
		g_eDenialEquipment[client][HE] = 1;
	else if(StrEqual(sItem, "smokegrenade"))
		g_eDenialEquipment[client][Smoke] = 1;
	else if(StrEqual(sItem, "defuser"))
		g_eDenialEquipment[client][Defusekit] = 1;
	else if(StrEqual(sItem, "nvgs"))
		g_eDenialEquipment[client][Nightvision] = 1;
}

Denial_OnGrenadeDetonate(client, String:sWeapon[])
{
	/* Reset player Denial data while Denial is disabled */
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hDenialConfig, ITEM_ENABLE))
	{
		Denial_ResetClient(client);
		return;
	}
	
	if(!StrContains(sWeapon, "hegrenade", false))
		g_eDenialEquipment[client][HE] = 0;
	else if(!StrContains(sWeapon, "flashbang", false))
		g_eDenialEquipment[client][FlashBang] = 0;
	else if(!StrContains(sWeapon, "smokegrenade", false))
		g_eDenialEquipment[client][Smoke] = 0;
}

bool:Denial_IsWeaponRestricted(String:sWeapon[])
{
	decl String:sRestrictedWeapons[1024];
	GetConVarString(g_hCVDenialRestrict, sRestrictedWeapons, sizeof(sRestrictedWeapons));
	
	decl String:sBuffer[64];
	strcopy(sBuffer, sizeof(sBuffer), sWeapon);
	ReplaceString(sBuffer, strlen(sBuffer)+1, "weapon_", "");
	
	if(StrContains(sRestrictedWeapons, sBuffer, false) != -1)
		return true;
	return false;
}

public Action:Denial_Timer_StripPlayer(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return Plugin_Stop;
	
	g_hDenialStripTimer[client] = INVALID_HANDLE;
	
	new iDenialLevel = GetPlayerItemLevel(client, GetArrayCell(g_hDenialConfig, ITEM_INDEX));
	
	if(iDenialLevel <= 0)
		return Plugin_Stop;
	
	// Level 1: Equipment
	if(iDenialLevel >= 1)
	{
		if(g_eDenialEquipment[client][HE] && !Client_HasWeapon(client, "weapon_hegrenade") && !Denial_IsWeaponRestricted("hegrenade"))
			GivePlayerItem(client, "weapon_hegrenade");
		if(g_eDenialEquipment[client][FlashBang] && !Client_HasWeapon(client, "weapon_flashbang") && !Denial_IsWeaponRestricted("flashbang"))
			GivePlayerItem(client, "weapon_flashbang");
		if(g_eDenialEquipment[client][Smoke] && !Client_HasWeapon(client, "weapon_smokegrenade") && !Denial_IsWeaponRestricted("smokegrenade"))
			GivePlayerItem(client, "weapon_smokegrenade");
		if(g_eDenialEquipment[client][Defusekit] && !GetEntProp(client, Prop_Send, "m_bHasDefuser") && !Denial_IsWeaponRestricted("defuser"))
			SetEntProp(client, Prop_Send, "m_bHasDefuser", 1);
		if(g_eDenialEquipment[client][Nightvision] && !GetEntProp(client, Prop_Send, "m_bHasNightVision") && !Denial_IsWeaponRestricted("nvgs"))
			SetEntProp(client, Prop_Send, "m_bHasNightVision", 1);
	}
	
	// Level 2: Secondary Weapon
	if(iDenialLevel >= 2)
	{
		if(!StrContains(g_sDenialSecondary[client],"weapon_") && !Denial_IsWeaponRestricted(g_sDenialSecondary[client]))
		{
			decl String:sOldWeapon[64];
			new iCurrentWeapon = GetPlayerWeaponSlot(client, 1);
			if(iCurrentWeapon != INVALID_ENT_REFERENCE)
			{
				GetEdictClassname(iCurrentWeapon, sOldWeapon, sizeof(sOldWeapon));
				Client_RemoveWeapon(client, sOldWeapon);
			}
			Client_GiveWeapon(client, g_sDenialSecondary[client]);
		}
	}
	
	// Level 3: Primary Weapon
	if(iDenialLevel >= 3)
	{
		if(!StrContains(g_sDenialPrimary[client],"weapon_") && !Denial_IsWeaponRestricted(g_sDenialPrimary[client]))
		{
			decl String:sOldWeapon[64];
			new iCurrentWeapon = GetPlayerWeaponSlot(client, 0);
			if(iCurrentWeapon != INVALID_ENT_REFERENCE)
			{
				GetEdictClassname(iCurrentWeapon, sOldWeapon, sizeof(sOldWeapon));
				Client_RemoveWeapon(client, sOldWeapon);
			}
			Client_GiveWeapon(client, g_sDenialPrimary[client]);
		}
	}
	
	return Plugin_Stop;
}