#pragma semicolon 1
#include <sourcemod>
#include <sdktools>

#define ICESTAB_INC 1.0 /* IceStab freeze duration increase for each level */
#define ICESTAB_DMG_MIN 50 /* Secondary knife attack is 50+ damage */
#define ICESTAB_CLRFADE 1 /* Blue color fade amount for each frame */

new Handle:g_hIceStabConfig;
new Handle:g_hCVIceStabLimitDmg;

new Handle:g_hIceStabUnfreeze[MAXPLAYERS+1] = {INVALID_HANDLE,...};
new g_iIceStabFade[MAXPLAYERS+1] = {255,...};

Item_IceStab_PreInit()
{
	g_hIceStabConfig = RegisterItemType("IceStab", "icestab", 10, true, 20, 30, 10, IceStab_Buy, IceStab_Sell);
	g_hCVIceStabLimitDmg = CreateConVar("cssrpg_icestab_limit_dmg", "10", "Maximum damage that can be done upon icestabbed victims (0 = disable)", 0, true, 0.0);
}

public bool:IceStab_Buy(client)
{
	return true;
}

public bool:IceStab_Sell(client)
{
	return true;
}

IceStab_ResetClient(client)
{
	if(g_hIceStabUnfreeze[client] != INVALID_HANDLE && IsClientInGame(client))
		TriggerTimer(g_hIceStabUnfreeze[client]);
	ClearHandle(g_hIceStabUnfreeze[client]);
	g_iIceStabFade[client] = 255;
}

IceStab_PlayerDamage(attacker, victim, dmg_health, dmg_armor)
{
	if((dmg_health+dmg_armor) < ICESTAB_DMG_MIN)
		return;
	
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hIceStabConfig, ITEM_ENABLE))
		return;
	
	IF_BOT_NENABLED(attacker)
		return;
	
	if(GetClientTeam(attacker) == GetClientTeam(victim))
		return;
	
	new iLevel = GetPlayerItemLevel(attacker, GetArrayCell(g_hIceStabConfig, ITEM_INDEX));
	
	if(iLevel <= 0)
		return;
	
	if(g_hIceStabUnfreeze[attacker] != INVALID_HANDLE)
		return; /* don't allow frozen attacker to icestab */
	
	SetEntityMoveType(victim, MOVETYPE_NONE);
	
	decl String:sSound[PLATFORM_MAX_PATH];
	Format(sSound, sizeof(sSound), "physics/glass/glass_impact_bullet%d.wav", GetRandomInt(1, 3));
	EmitSoundToAll(sSound, SOUND_FROM_PLAYER, SNDCHAN_AUTO, SNDLEVEL_NORMAL, SND_NOFLAGS, SNDVOL_NORMAL, SNDPITCH_NORMAL, victim);

	SetEntityRenderMode(victim, RENDER_TRANSCOLOR);
	Entity_SetRenderColor(victim, 0, 0, 255, -1);
	
	ClearHandle(g_hIceStabUnfreeze[victim]);
	g_hIceStabUnfreeze[victim] = CreateTimer(float(iLevel), IceStab_Timer_Unfreeze, GetClientUserId(victim), TIMER_FLAG_NO_MAPCHANGE);
	g_iIceStabFade[victim] = 0;
}

IceStab_LimitDamage(victim, &dmg_health, String:weapon[])
{
	if(g_hIceStabUnfreeze[victim] == INVALID_HANDLE)
		return;
	
	new iLimitDmg = GetConVarInt(g_hCVIceStabLimitDmg);
	if(iLimitDmg <= 0)
		return;
	
	if(StrContains(weapon, "knife", false) == -1)
		return;
	
	if(dmg_health <= iLimitDmg)
		return;
	
	new iOldHP = GetClientHealth(victim) + dmg_health;
	
	dmg_health = iLimitDmg;
	
	SetEntityHealth(victim, iOldHP-dmg_health);
	
	decl String:sSound[PLATFORM_MAX_PATH];
	Format(sSound, sizeof(sSound), "physics/glass/glass_sheet_impact_hard%d.wav", GetRandomInt(1, 3));
	EmitSoundToAll(sSound, SOUND_FROM_PLAYER, SNDCHAN_AUTO, SNDLEVEL_NORMAL, SND_NOFLAGS, SNDVOL_NORMAL, SNDPITCH_NORMAL, victim);
}

public Action:IceStab_Timer_Unfreeze(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	if(!client)
		return Plugin_Stop;
	
	g_hIceStabUnfreeze[client] = INVALID_HANDLE;
	SetEntityMoveType(client, MOVETYPE_WALK);
	
	return Plugin_Stop;
}

public IceStab_OnGameFrame()
{
	for(new i=1;i<=MaxClients;i++)
	{
		if(!IsClientInGame(i))
			continue;
		
		if(g_iIceStabFade[i] < 255)
		{
			g_iIceStabFade[i] += ICESTAB_CLRFADE;
			SetEntityRenderMode(i, RENDER_TRANSCOLOR);
			Entity_SetRenderColor(i, g_iIceStabFade[i], g_iIceStabFade[i], 255, -1);
		}
	}
}