#pragma semicolon 1
#include <sourcemod>
#include <sdktools>

/**
 * @brief Heal increment for each level.
 */
#define MEDIC_INC 5

/**
 * @brief Delay between each heal.
 */
#define MEDIC_DELAY 2.0

/**
 * @brief Medic healing radius.
 */
#define MEDIC_RADIUS 250.0

#define MEDIC_HEALTH_BEAM_COLOR {5, 45, 255, 50}
#define MEDIC_ARMOR_BEAM_COLOR {5, 255, 10, 50}

new Handle:g_hMedicConfig;

new Handle:g_hMedicTimer = INVALID_HANDLE;

Item_Medic_PreInit()
{
	g_hMedicConfig = RegisterItemType("Medic", "medic", 20, true, 15, 15, 20, Medic_Buy, Medic_Sell);
}

Medic_Init()
{
	//if(!GetArrayCell(g_hMedicConfig, ITEM_ENABLE))
	//	return;
	
	g_hMedicTimer = CreateTimer(MEDIC_DELAY, Medic_Timer_ApplyMedic, _, TIMER_REPEAT|TIMER_FLAG_NO_MAPCHANGE);
}

Medic_Shutdown()
{
	ClearHandle(g_hMedicTimer);
}

public bool:Medic_Buy(client)
{
	return true;
}

public bool:Medic_Sell(client)
{
	return true;
}

/**
 * @brief Checks the distance of each player from a medic and assigns health
 *        to them accordingly.
 */
public Action:Medic_Timer_ApplyMedic(Handle:timer, any:data)
{
	if(!GetConVarBool(g_hCVEnable) || !GetArrayCell(g_hMedicConfig, ITEM_ENABLE))
		return Plugin_Continue;
	
	new bool:bBotEnable = GetConVarBool(g_hCVBotEnable);
	
	// Build origin cache and team targets for beam rings
	decl Float:vCacheOrigin[MaxClients+1][3];
	decl iTerrorTeam[MaxClients+1], iCTTeam[MaxClients+1];
	new iTerrorCount, iCTCount;
	for(new i=1;i<=MaxClients;i++)
	{
		if(!IsClientInGame(i))
			continue;
		
		if(!bBotEnable && IsFakeClient(i))
			continue;
		
		if(GetClientTeam(i) == 2)
			iTerrorTeam[iTerrorCount++] = i;
		else if(GetClientTeam(i) == 3)
			iCTTeam[iCTCount++] = i;
		
		if(!IsPlayerAlive(i))
			continue;
		
		GetClientEyePosition(i, vCacheOrigin[i]);
	}
	
	new bool:bMedicDidHisJob, iBeamRingColor[4];
	
	decl iMedicLevel, iNewHP, iNewArmor, Float:vRingOrigin[3];
	for(new i=1;i<=MaxClients;i++)
	{
		/* If player is a medic and player is not dead */
		if(!IsClientInGame(i) || !IsPlayerAlive(i))
			continue;
		
		if(!bBotEnable && IsFakeClient(i))
			continue;
		
		iMedicLevel = GetPlayerItemLevel(i, GetArrayCell(g_hMedicConfig, ITEM_INDEX));
		if(iMedicLevel <= 0)
			continue;
		
		/* Medic found, now search for teammates */
		for(new m=1;m<=MaxClients;m++)
		{
			/* If player is on the same team as medic, player is not dead,
				   and player is not the medic */
			if(m == i || !IsClientInGame(m) || !IsPlayerAlive(m) || GetClientTeam(m) != GetClientTeam(i))
				continue;
			
			if(!bBotEnable && IsFakeClient(m))
				continue;
			
			/* A suitable player has been found */
			/* Check if player is in the medic's radius */
			if(GetVectorDistance(vCacheOrigin[i], vCacheOrigin[m]) > MEDIC_RADIUS)
				continue;
			
			iNewHP = GetClientHealth(m);
			iNewArmor = GetClientArmor(m);
			
			bMedicDidHisJob = false;
			
			/* If player is not at maximum health, heal him */
			if(iNewHP < HealthP_GetMaxHealth(m))
			{
				iNewHP += iMedicLevel * MEDIC_INC;
				
				if(iNewHP > HealthP_GetMaxHealth(m))
					iNewHP = HealthP_GetMaxHealth(m);
				
				SetEntityHealth(m, iNewHP);
				
				iBeamRingColor = MEDIC_HEALTH_BEAM_COLOR;
				
				bMedicDidHisJob = true;
			}
			/* Else if player is not at maximum armor, repair him */
			else if(iNewArmor < 100)
			{
				if(iMedicLevel*MEDIC_INC > 25)
					iNewArmor += 25;
				else
					iNewArmor += iMedicLevel*MEDIC_INC;
				
				if(iNewArmor > 100)
					iNewArmor = 100;
				
				SetEntProp(m, Prop_Send, "m_ArmorValue", iNewArmor);
				
				iBeamRingColor = MEDIC_ARMOR_BEAM_COLOR;
				
				bMedicDidHisJob = true;
			}
			
			// Only run the effects, if something happended
			if(!bMedicDidHisJob)
				continue;
			
			vRingOrigin = vCacheOrigin[i];
			vRingOrigin[2] -= 25.0;
			
			TE_SetupBeamRingPoint(vRingOrigin, 8.0, MEDIC_RADIUS+300.0, g_iBeamRingSprite, g_iBeamRingSprite, 0, 1, 1.5, 10.0, 0.0, iBeamRingColor, 0, FBEAM_FADEOUT);
			
			if(GetClientTeam(i) == 2)
				TE_Send(iTerrorTeam, iTerrorCount);
			else
				TE_Send(iCTTeam, iCTCount);
			
			EmitSoundToAll("weapons/physcannon/physcannon_charge.wav", SOUND_FROM_PLAYER, SNDCHAN_AUTO, SNDLEVEL_NORMAL, SND_NOFLAGS, 0.2, SNDPITCH_NORMAL, i);
		}
	}
	
	return Plugin_Continue;
}